///
/// A Testing TreeModel that generates a deep tree of simply labeled nodes. 
/// Implements the TreeModel interface.  This tree consists of three classes:
///   * TestSystemTreeModel  - The main tree model
///   * TestSystemTreeNode   - How individual nodes directories are represented
///   * TestSystemHeaderNode - How the column header is represented
/// 
/// Ben Bederson
/// Human-Computer Interaction Lab
/// University of Maryland
/// May, 2005
/// 
using System;
using TreeControl;

namespace TreeControlTester {
	public class TestSystemTreeModel : TreeControl.TreeModel {
		public const int NUM_NODES = 5;
		public const int NUM_LEAVES = 5;
		public const int NUM_COLS = 5;

		public TestSystemTreeModel() {
		}

		public TreeNode GetChild(TreeNode parent, int index) {
			TreeNode child = null;

			if (parent is TestSystemTreeNode) {
				TestSystemTreeNode testNode = (TestSystemTreeNode)parent;

				string path;
				if (testNode.Path.StartsWith("Root")) {
					path = index.ToString();
				} else {
					path = testNode.Path + "." + index;
				}
				bool isFolder;
				if (index < NUM_NODES) {
					isFolder = true;
				} else {
					isFolder = false;
				}
				child = new TestSystemTreeNode(path, isFolder);
			}

			return child;
		}

		public int GetChildCount(TreeNode parent) {
			return NUM_NODES + NUM_LEAVES;
		}

		public int GetIndexOfChild(TreeNode parent, TreeNode child) {
			int index = -1;

			for (int i=0; i<GetChildCount(parent); i++) {
				if (GetChild(parent, i).Equals(child)) {
					index = i;
					break;
				}
			}

			return index;
		}

		public TreeNode GetRoot() {
			return new TestSystemTreeNode("Root", true);
		}

		public bool IsLeaf(TreeNode node) {
			bool isLeaf = true;

			if (node is TestSystemTreeNode) {
				TestSystemTreeNode testNode = node as TestSystemTreeNode;
				if (testNode.IsFolder) {
					isLeaf = false;
				}
			}

			return isLeaf;
		}

		public TreeNode GetColumnHeader() {
			return new TestSystemHeaderNode();
		}
	}

	public class TestSystemTreeNode : TreeControl.TreeNode {
		private string path;
		private bool folder;

		public TestSystemTreeNode(string path, bool folder) {
			this.path = path;
			this.folder = folder;
		}

		public int GetColumnCount() {
			return TestSystemTreeModel.NUM_COLS;
		}

		public string GetColumn(int index) {
			if (index == 0) {
				return path;
			} else {
				return "Col " + index;
			}
		}

		public bool IsFolder {
			get { return folder; }
		}

		public string Path {
			get { return path; }
		}

		public override bool Equals(object obj) {
			if (obj is TestSystemTreeNode) {
				if (path.Equals((obj as TestSystemTreeNode).Path)) {
					return true;
				}
			}
			return false;
		}

		public override int GetHashCode() {
			return base.GetHashCode ();
		}
	}

	public class TestSystemHeaderNode : TreeNode {
		public TestSystemHeaderNode() {
		}

		public int GetColumnCount() {
			return TestSystemTreeModel.NUM_COLS;
		}

		public string GetColumn(int index) {
			if (index == 0) {
				return "Column " + index + "                   ";
			} else {
				return "Column " + index;
			}
		}
	}
}